#include <cstdlib>
#include <string>
#include <fstream>
#include "globals.h"
#include "example_set.h"
#include "svm_c.h"
#include "svm_nu.h"
#include "parameters.h"
#include "kernel.h"
#include "version.h"
#include <windows.h>

HWND hwndEditPattern ;
HWND hwndEditRegression ;
HWND hwndEditC ;
HWND hwndEditE ;
HWND hwndEditE1 ;
HWND hwndEditNu ;
HWND hwndEditNu1 ;
HWND hwndEditD ;
HWND hwndEditP ;
HWND hwndEditP1 ;
HWND hwndEditR ;
HWND hwndEditR1 ;
HWND hwndEditN ;
HWND hwndEditN1 ;
HWND hwndEditN2 ;
HWND hwndEditA ;
HWND hwndEditA1 ;
HWND hwndEditA2 ;

//ofstream output_stream;

// global svm-objects
kernel_c* kernelp=0;
parameters_c* parametersp=0;
svm_c* svmp;
example_set_c* training_setp=0;
int is_linearp=1; // linear kernel?

string kp[3];

struct example_set_list{
  example_set_c* the_set;
  example_set_list* next;
};
example_set_list* test_setsp = 0;


void print_help_p(){
  output_stream<<endl;
  output_stream<<"predict: predict a set of examples with a trained SVM."<<endl<<endl;
  output_stream<<"usage: predict"<<endl
      <<"       predict <FILE>"<<endl
      <<"       predict <FILE1> <FILE2> ..."<<endl<<endl;
  output_stream<<"The input has to consist of:"<<endl
      <<"- the svm parameters"<<endl
      <<"- the kernel definition"<<endl
      <<"- the training result set"<<endl
      <<"- one or more sets to predict"<<endl;

  output_stream<<endl<<"See the documentation for the input format. The first example set to be entered is considered to be the training set, all others are test sets. Each input file can consist of one or more definitions. If no input file is specified, the input is read from <stdin>."<<endl<<endl;

  output_stream<<endl<<"This software is free only for non-commercial use. It must not be modified and distributed without prior permission of the author. The author is not responsible for implications from the use of this software."<<endl;
  exit(0);
};


void read_inputp(){

kernel_container_c k_cont;

//char* filename;
ifstream input_stream;
input_stream.open(InputFilename, fstream::in);
    if(!input_stream) {
        MessageBox(NULL, InputFilename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
//strcpy(filename, InputFilename); //PROBLEM
  // returns number of examples sets read
  char* s = new char[MAXCHAR];
  char next;
  next = input_stream.peek();
  if(next == EOF){ 
    // set stream to eof
    next = input_stream.get(); 
  };
  while(! input_stream.eof()){
    if('#' == next){
      // ignore comment
      input_stream.getline(s,MAXCHAR);
    }
    else if('\n' == next){
      // ignore newline
      next = input_stream.get();
    }
    else if('@' == next){
      // new section
      input_stream >> s;
      if(0==strcmp("@parameters",s)){
	// read parameters
	if(parametersp == 0){
	  parametersp = new parameters_c();
	  input_stream >> *parametersp;
	}
	else{
	  output_stream <<"*** ERROR: Parameters multiply defined"<<endl;
	  throw input_exception();
	};
      }
      else if(0==strcmp("@examples",s)){
	if(0 == training_setp){
	  // input training set
	  training_setp = new example_set_c();
	  if(0 != parametersp){
	    training_setp->set_format(parametersp->default_example_format);
	  };
	  input_stream  >> *training_setp;	    
	  training_setp->set_filename(InputFilename);
	  output_stream<<"   read "<<training_setp->size()<<" examples, format "<<training_setp->my_format<<", dimension = "<<training_setp->get_dim()<<"."<<endl;
	}
	else{
	  // input test sets
	  example_set_list* test_set = new example_set_list;
	  test_set->the_set = new example_set_c();
	  if(0 != parametersp){
	    (test_set->the_set)->set_format(parametersp->default_example_format);
	  };
	  input_stream >> *(test_set->the_set);
	  (test_set->the_set)->set_filename(InputFilename);
	  test_set->next = test_setsp;
	  test_setsp = test_set;
	  output_stream<<"   read "<<(test_set->the_set)->size()<<" examples, format "<<(test_set->the_set)->my_format<<", dimension = "<<(test_set->the_set)->get_dim()<<"."<<endl;
	};
      }
      else if(0==strcmp("@kernel",s)){
	if(0 == kernelp){
	  kernel_container_c k_cont;
	  input_stream >> k_cont;
	  kernelp = k_cont.get_kernel();
	}
	else{
	  output_stream <<"*** ERROR: Kernel multiply defined"<<endl;
	  throw input_exception();
	};
      };
    }
    else{
      // default = "@examples"
      if(0 == training_setp){
	// input training set
	training_setp = new example_set_c();
	if(0 != parametersp){
	  training_setp->set_format(parametersp->default_example_format);
	};
	input_stream  >> *training_setp;	    
	training_setp->set_filename(InputFilename);
	output_stream<<"   read "<<training_setp->size()<<" examples, format "<<training_setp->my_format<<", dimension = "<<training_setp->get_dim()<<"."<<endl;
      }
      else{
	// input test sets
	example_set_list* test_set = new example_set_list;
	test_set->the_set = new example_set_c();
	if(0 != parametersp){
	  (test_set->the_set)->set_format(parametersp->default_example_format);
	};
	input_stream >> *(test_set->the_set);
	(test_set->the_set)->set_filename(InputFilename);
	test_set->next = test_setsp;
	test_setsp = test_set;
	output_stream<<"   read "<<(test_set->the_set)->size()<<" examples, format "<<(test_set->the_set)->my_format<<", dimension = "<<(test_set->the_set)->get_dim()<<"."<<endl;
      };
    };
    next = input_stream.peek();
    if(next == EOF){ 
      // set stream to eof
      next = input_stream.get(); 
    };
  };
  delete []s;
  
  
  
    //mvs hack
    ifstream kernel_stream;
    ifstream kernel_stream2;


    kernel_stream.open(kernel_filename, fstream::in);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream >> k_cont;
    kernelp = k_cont.get_kernel();
    is_linearp = k_cont.is_linear;
    kernel_stream.close();


    kernel_stream2.open(kernel_filename, fstream::in);
    if(!kernel_stream2) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kp[0] = "";
    kp[1] = "";
    kp[2] = "";
    int ki = 0;
    string line;
    while (getline(kernel_stream2, line)) {
        kp[ki++] = line;
    }
    kernel_stream2.close();
//end Martin's hack

  
  
  
  
  
  
  
  input_stream.close();
};





int predict(){
    
    
kernelp=0;
parametersp=0;
training_setp=0;
is_linearp=1; // linear kernel?
test_setsp = 0;
    

    
    output_stream.open (output_filename, fstream::trunc);
    
    if(!output_stream) {
        MessageBox(NULL, output_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }

ofstream test_stream;
fstream kernel_stream;
ifstream kernel_stream2;

    test_stream.open(kernel_filename, fstream::trunc);
    if(!test_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    test_stream.close();
    
char OutputFilenamepred[256];

  if(!InputFilename[0])
            MessageBox(NULL, "No training file selected", "Error", MB_OK | MB_ICONINFORMATION);
  else {




//Martin's hack
int iLength;
PTSTR pstrDoc;
    char OutputFilenamesvm[256];
    char OutputFilenamepred[256];


if (SendMessage(hwndEditPattern,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_pattern = true;
else
    windows_is_pattern = false;

if (SendMessage(hwndEditRegression,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_regression = true;
else
    windows_is_regression = false;



//C
iLength = GetWindowTextLength (hwndEditC);
if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
    return false;
GetWindowText (hwndEditC, pstrDoc, iLength + 1) ;
  if(iLength > 0)
      windows_is_C = true;
  else
      windows_is_C = false;
windows_C = atof(pstrDoc);
free(pstrDoc);



//Epsilon
if (SendMessage(hwndEditE,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_epsilon = true;
else
    windows_is_epsilon = false;

    iLength = GetWindowTextLength (hwndEditE1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditE1, pstrDoc, iLength + 1) ;
    windows_epsilon = atof(pstrDoc);
    free(pstrDoc);



//Nu

if (SendMessage(hwndEditNu,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_nu = true;
else
    windows_is_nu = false;


    iLength = GetWindowTextLength (hwndEditNu1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditNu1, pstrDoc, iLength + 1) ;
    windows_nu = atof(pstrDoc);
    free(pstrDoc);









    //Dot
if (SendMessage(hwndEditD,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type dot" << endl;
    kernel_stream.close();
}



    //Polynomial
if (SendMessage(hwndEditP,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type polynomial" << endl;
    iLength = GetWindowTextLength (hwndEditP1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditP1, pstrDoc, iLength + 1) ;
    kernel_stream << "degree " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}




//Radial
if (SendMessage(hwndEditR,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type radial" << endl;
    iLength = GetWindowTextLength (hwndEditR1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditR1, pstrDoc, iLength + 1) ;
    kernel_stream << "gamma " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}



//Neural
if (SendMessage(hwndEditN,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type neural" << endl;
    iLength = GetWindowTextLength (hwndEditN1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditN1, pstrDoc, iLength + 1) ;
    kernel_stream << "a " << pstrDoc << endl;
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditN2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditN2, pstrDoc, iLength + 1) ;
    kernel_stream << "b " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}


//Anova
if (SendMessage(hwndEditA,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type anova" << endl;
    iLength = GetWindowTextLength (hwndEditA1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditA1, pstrDoc, iLength + 1) ;
    kernel_stream << "gamma " << pstrDoc << endl;
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditA2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditA2, pstrDoc, iLength + 1) ;
    kernel_stream << "degree " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}








  output_stream<<"*** mySVM version "<<mysvmversion<<" ***"<<endl;
  // read objects
  try{
 	    output_stream<<"Reading "<<InputFilename<<endl;
	    ifstream input_file(InputFilename);
	    if(input_file.bad()){
	      output_stream<<"ERROR: Could not read file \""<<InputFilename<<"\", exiting."<<endl;
	      MessageBox(NULL, InputFilename, "ERROR: Could not read file ", MB_OK | MB_ICONINFORMATION);
	      exit(1);
	    };
	    read_inputp();
	    input_file.close();

  }
  catch(general_exception &the_ex){
    output_stream<<"*** Error while reading input: "<<the_ex.error_msg<<endl;
    MessageBox(NULL, the_ex.error_msg, "*** Error while reading input: ", MB_OK | MB_ICONINFORMATION);
    exit(1);
  }
  
  

  
  catch(...){
    output_stream<<"*** Program ended because of unknown error while reading input"<<endl;
    MessageBox(NULL, "Error", "*** Program ended because of unknown error while reading input", MB_OK | MB_ICONINFORMATION);
    exit(1);
  };

  if(0 == parametersp){
    parametersp = new parameters_c();
    if(training_setp->initialised_pattern_y()){
      parametersp->is_pattern = 1;
      parametersp->do_scale_y = 0;
    };
  };
  if(0 == kernelp){
    kernelp = new kernel_dot_c();
  };
  if(0 == training_setp){
    output_stream << "*** ERROR: You did not enter the training set"<<endl;
    MessageBox(NULL, "Error", "*** ERROR: You did not enter the training set", MB_OK | MB_ICONINFORMATION);
    exit(1);
  };

  if(parametersp->is_distribution){
    svmp = new svm_distribution_c();
  }
  else if(parametersp->is_nu){
    if(parametersp->is_pattern){
      svmp = new svm_nu_pattern_c();
    }
    else{
      svmp = new svm_nu_regression_c();
    };
  }
  else if(parametersp->is_pattern){
    svmp = new svm_pattern_c();
  }
  else{
    svmp = new svm_regression_c();
  };

  // scale examples
  if(parametersp->do_scale){
    training_setp->scale(parametersp->do_scale_y);
  };

  kernelp->init(parametersp->kernel_cache,training_setp);
  svmp->init(kernelp,parametersp);
  svmp->set_svs(training_setp);

  // testing
  if(0 != test_setsp){
    output_stream<<"----------------------------------------"<<endl;
    output_stream<<"Predicting"<<endl;
    example_set_c* next_test;
    SVMINT test_no = 0;
    //char* outname = new char[MAXCHAR];
    while(test_setsp != 0){
      test_no++;
      next_test = test_setsp->the_set;
      if(training_setp->initialised_scale()){
	next_test->scale(training_setp->get_exp(),
			 training_setp->get_var(),
			 training_setp->get_dim());
      };
      if(next_test->initialised_y()){
	output_stream<<"Testing examples from file "<<(next_test->get_filename())<<endl;
	svmp->test(next_test,1);
      };
      output_stream<<"Predicting examples from file "<<(next_test->get_filename())<<endl;
      svmp->predict(next_test);
      // output to file .pred
      

    int length = strlen(InputFilename);
    
    strcpy(OutputFilenamepred, InputFilename);
    OutputFilenamepred[length - 4] = '-';
    OutputFilenamepred[length - 3] = 'p';
    OutputFilenamepred[length - 2] = 'r';
    OutputFilenamepred[length - 1] = 'e';
    OutputFilenamepred[length + 0] = 'd';
    OutputFilenamepred[length + 1] = InputFilename[length -4];
    OutputFilenamepred[length + 2] = InputFilename[length -3];
    OutputFilenamepred[length + 3] = InputFilename[length -2];
    OutputFilenamepred[length + 4] = InputFilename[length -1];
    OutputFilenamepred[length + 5] = '\0';
      

      //strcpy(outname,next_test->get_filename());
      //strcat(outname,".pred");
      ofstream output_file(OutputFilenamepred,ios::out|ios::trunc);
      next_test->output_ys(output_file);
      output_file.close();	
      output_stream<<"Prediction saved in file "<<OutputFilenamepred<<endl;
      test_setsp = test_setsp->next; // skip delete!
    };
    //delete []outname;
  };

  if(parametersp->verbosity > 1){
    output_stream << "mysvm ended successfully."<<endl;
  };
  output_stream.close();
  MessageBox(NULL, "Finished", "winSVM", MB_OK | MB_ICONINFORMATION);
}  
  return(0);
};
