#include <cstdlib>
#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <cfloat>
#include <ctime>
#include <sys/types.h>
#include <unistd.h>

#include "globals.h"
#include "example_set.h"
#include "svm_c.h"
#include "parameters.h"
#include "kernel.h"
#include "svm_nu.h"
#include "version.h"
#include "resource.h"
#include <windows.h>

using namespace std;

int learn();
int predict();
int optimize();

int  GetOpenFilename(char *, char *);

char DefaultInputFile[256] = "input.txt";
char InputFilename[256] = "input.txt";

ifstream input_file;
ofstream output_stream;

HWND hWndButtonI;
HWND hWndButtonO;
HWND hWndButtonLearn;
HWND hWndButtonPredict;
HWND hWndButtonOptimize;




extern HWND hwndEditPattern ;
extern HWND hwndEditRegression ;
extern HWND hwndEditC ;
extern HWND hwndEditE ;
extern HWND hwndEditE1 ;
extern HWND hwndEditNu ;
extern HWND hwndEditNu1 ;
extern HWND hwndEditD ;
extern HWND hwndEditP ;
extern HWND hwndEditP1 ;
extern HWND hwndEditR ;
extern HWND hwndEditR1 ;
extern HWND hwndEditN ;
extern HWND hwndEditN1 ;
extern HWND hwndEditN2 ;
extern HWND hwndEditA ;
extern HWND hwndEditA1 ;
extern HWND hwndEditA2 ;

HWND hwndEditRuns;
HWND hwndEditCounter;

static HWND hwndEditInputFilename ;
static HWND hwndEditOutputFilename ;




bool windows_is_C;
SVMFLOAT windows_C;
bool windows_is_epsilon;
SVMFLOAT windows_epsilon;
bool windows_is_nu;
SVMFLOAT windows_nu;
bool windows_is_pattern;
bool windows_is_regression;





extern SVMFLOAT global_MSE;
SVMFLOAT global_MSE;
const char* kernel_filename = "kernel.txt";
const char* output_filename = "output.txt";


string k[3];
char perc[5];




// global svm-objects
kernel_c* kernel=0;
parameters_c* parameters=0;
svm_c* svm;
example_set_c* training_set=0;
int is_linear=1; // linear kernel?

struct example_set_list{
  example_set_c* the_set;
  example_set_list* next;
};
example_set_list* test_sets = 0;


//Martin's hack
int random_range(int lowest_number, int highest_number)
{
    if(lowest_number > highest_number)
        swap(lowest_number, highest_number);
    int range = highest_number - lowest_number + 1;
    return lowest_number + int(range * rand()/(RAND_MAX + 1.0));
}

void print_help(){
  output_stream<<endl;
  output_stream<<"my_svm: train a svm from the given parameters and examples."<<endl<<endl;
  output_stream<<"usage: my_svm"<<endl
      <<"       my_svm <FILE>"<<endl
      <<"       my_svm <FILE1> <FILE2> ..."<<endl<<endl;
  output_stream<<"The input has to consist of:"<<endl
      <<"- the svm parameters"<<endl
      <<"- the kernel definition"<<endl
      <<"- the training set"<<endl
      <<"- one or more test sets (optional)"<<endl;

  output_stream<<endl<<"See the documentation for the input format. The first example set to be entered is considered to be the training set, all others are test sets. Each input file can consist of one or more definitions. If no input file is specified, the input is read from <stdin>."<<endl<<endl;

  output_stream<<endl<<"This software is free only for non-commercial use. It must not be modified and distributed without prior permission of the author. The author is not responsible for implications from the use of this software."<<endl;
  exit(0);
};


void read_input(){
  // returns number of examples sets read

  ifstream input_stream;
  input_stream.open (InputFilename, fstream::in);
  kernel_container_c k_cont;
  char* s = new char[MAXCHAR];
  char next;
  
  
  
 

  next = input_stream.peek();
  if(next == EOF){
    // set stream to eof
    next = input_stream.get();
  };
  while(! input_stream.eof()){
    if('#' == next){
      // ignore comment
      input_stream.getline(s,MAXCHAR);
    }
    else if(('\n' == next) ||
	    (' ' == next) ||
	    ('\r' == next) ||
	    ('\f' == next) ||
	    ('\t' == next)){
      // ignore
      next = input_stream.get();
    }
    else if('@' == next){
      // new section
      input_stream >> s;
      if(0 == strcmp("@parameters",s)){
   
  	// read parameters
	if(parameters == 0){
	  parameters = new parameters_c();
	  input_stream >> *parameters;
	}
	else{
	  cout <<"*** ERROR: Parameters multiply defined"<<endl;
	  throw input_exception();
	};

      }
      else if(0==strcmp("@examples",s)){
	if(0 == training_set){
	  // input training set
	  training_set = new example_set_c();
	  if(0 != parameters){
	    training_set->set_format(parameters->default_example_format);
	  };
	  input_stream >> *training_set;
	  training_set->set_filename(InputFilename);
	  cout<<"   read "<<training_set->size()<<" examples, format "<<training_set->my_format<<", dimension = "<<training_set->get_dim()<<"."<<endl;
	}
	else{
	  // input test sets
	  example_set_list* test_set = new example_set_list;
	  test_set->the_set = new example_set_c();
	  if(0 != parameters){
	    (test_set->the_set)->set_format(parameters->default_example_format);
	  };
	  input_stream >> *(test_set->the_set);
	  (test_set->the_set)->set_filename(InputFilename);
	  test_set->next = test_sets;
	  test_sets = test_set;
	  cout<<"   read "<<(test_set->the_set)->size()<<" examples, format "<<(test_set->the_set)->my_format<<", dimension = "<<(test_set->the_set)->get_dim()<<"."<<endl;
	};
      }
      else if(0==strcmp("@kernel",s)){
	if(0 == kernel){
	  //kernel_container_c k_cont; mvs
	  input_stream >> k_cont;
	  kernel = k_cont.get_kernel();
	  is_linear = k_cont.is_linear;
	}
	else{
	  cout <<"*** ERROR: Kernel multiply defined"<<endl;
	  throw input_exception();
	};
      };
    }
    else{
      // default = "@examples"
      if(0 == training_set){
	// input training set
	training_set = new example_set_c();
	if(0 != parameters){
	  training_set->set_format(parameters->default_example_format);
	};
	input_stream  >> *training_set;
	training_set->set_filename(InputFilename);
	cout<<"   read "<<training_set->size()<<" examples, format "<<training_set->my_format<<", dimension = "<<training_set->get_dim()<<"."<<endl;
      }
      else{
	// input test sets
	example_set_list* test_set = new example_set_list;
	test_set->the_set = new example_set_c();
	if(0 != parameters){
	  (test_set->the_set)->set_format(parameters->default_example_format);
	};
	input_stream >> *(test_set->the_set);
	(test_set->the_set)->set_filename(InputFilename);
	test_set->next = test_sets;
	test_sets = test_set;
	cout<<"   read "<<(test_set->the_set)->size()<<" examples, format "<<(test_set->the_set)->my_format<<", dimension = "<<(test_set->the_set)->get_dim()<<"."<<endl;
      };
    };
    next = input_stream.peek();
    if(next == EOF){
      // set stream to eof
      next = input_stream.get();
    };
  };
  delete []s;


  //mvs hack
    ifstream kernel_stream;
    ifstream kernel_stream2;


    kernel_stream.open(kernel_filename, fstream::in);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream >> k_cont;
    kernel = k_cont.get_kernel();
    is_linear = k_cont.is_linear;
    kernel_stream.close();


    kernel_stream2.open(kernel_filename, fstream::in);
    if(!kernel_stream2) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    k[0] = "";
    k[1] = "";
    k[2] = "";
    int ki = 0;
    string line;
    while (getline(kernel_stream2, line)) {
        k[ki++] = line;
    }
    kernel_stream2.close();
//end Martin's hack

};


svm_result do_cv(){
  SVMINT number = parameters->cross_validation;
  SVMINT size = training_set->size();
  int verbosity = parameters->verbosity;
  if((number > size) || (0 >= number)){
    number = size; // leave-one-out testing
  };
  //  SVMINT cv_size = size / number;
  if(! parameters->cv_inorder){
    training_set->permute();
  };
  training_set->clear_alpha();
  example_set_c* cv_train=new example_set_c(); //=0;
  example_set_c* cv_test=new example_set_c(); //=0;
  svm_result train_result;
  svm_result test_result;
  svm_result train_sum;
  svm_result test_sum;
  train_sum.VCdim = 0;
  train_sum.pred_loss=0;
  train_sum.loss=0;
  train_sum.loss_pos=0;
  train_sum.loss_neg=0;
  train_sum.MAE = 0;
  train_sum.MSE = 0;
  train_sum.accuracy = 0;
  train_sum.precision = 0;
  train_sum.recall=0;
  train_sum.number_svs=0;
  train_sum.number_bsv=0;
  test_sum.VCdim = 0;
  test_sum.loss=0;
  test_sum.loss_pos=0;
  test_sum.loss_neg=0;
  test_sum.MAE = 0;
  test_sum.MSE = 0;
  test_sum.accuracy = 0;
  test_sum.precision = 0;
  test_sum.recall=0;
  test_sum.number_svs=0;
  test_sum.number_bsv=0;
  SVMINT j;
  if(verbosity>2){
    if(parameters->cv_window>0){
      output_stream<<"beginning "<<(number-parameters->cv_window)<<" sliding window steps"<<endl;
    }
    else{
      output_stream<<"beginning "<<number<<"-fold crossvalidation"<<endl;
    };
  };
  SVMINT i;
  for(i=parameters->cv_window;i<number;i++){
    // do cv
    if(verbosity >= 3){
      output_stream<<"----------------------------------------"<<endl;
      output_stream<<(i+1);
      if(0 == i%10) output_stream<<"st";
      else if(1==i%10) output_stream<<"nd";
      else if(2==i%10) output_stream<<"rd";
      else output_stream<<"th";
      output_stream<<" step"<<endl;
    };
    //output_stream<<"From "<<i*cv_size<<" to "<<(i+1)*cv_size<<endl;
    cv_train->clear();
    cv_test->clear();
    cv_train->set_dim(training_set->get_dim());
    cv_test->set_dim(training_set->get_dim());
    if(training_set->initialised_y()){
      cv_train->set_initialised_y();
      cv_test->set_initialised_y();
    };
    cv_train->put_Exp_Var(training_set->get_exp(),training_set->get_var());
    cv_test->put_Exp_Var(training_set->get_exp(),training_set->get_var());
    if(verbosity>=4){
      output_stream<<"Initing examples sets"<<endl;
    };

    if(parameters->cv_window>0){
      // training window
      for(j=SVMINT((i-parameters->cv_window)*size/number);((j<(SVMINT)(i*size/number))&&(j<size));j++){
	cv_train->put_example(training_set->get_example(j));
      };
      // test window
      for(j=(SVMINT)(i*size/number);((j<(SVMINT)((i+1)*size/number))&&(j<size));j++){
	cv_test->put_example(training_set->get_example(j));
      };
    }
    else{
      for(j=(SVMINT)(i*size/number);((j<(SVMINT)((i+1)*size/number))&&(j<size));j++){
	cv_test->put_example(training_set->get_example(j));
      };
      for(j=0;j<(SVMINT)(i*size/number);j++){
	cv_train->put_example(training_set->get_example(j));
      };
      for(j=(SVMINT)((i+1)*size/number);j<size;j++){
	cv_train->put_example(training_set->get_example(j));
      };
    };
    cv_train->clear_alpha();
    cv_test->clear_alpha();
    cv_train->compress();
    cv_test->compress();

    if(verbosity>=4){
      output_stream<<"Setting up the SVM"<<endl;
    };
    kernel->init(parameters->kernel_cache,cv_train);
    svm->init(kernel,parameters);

    // train & test the svm
    if(verbosity>=4){
      output_stream<<"Training"<<endl;
    };
    //    cv_train->clear_alpha();

    train_result = svm->train(cv_train);
    if(verbosity>=4){
      output_stream<<"Testing"<<endl;
    };
    test_result = svm->test(cv_test,0);

    train_sum.VCdim += train_result.VCdim;
    train_sum.loss += train_result.loss;
    train_sum.loss_pos += train_result.loss_pos;
    train_sum.loss_neg += train_result.loss_neg;
    train_sum.MAE += train_result.MAE;
    train_sum.MSE += train_result.MSE;
    train_sum.pred_loss += train_result.pred_loss;
    train_sum.accuracy += train_result.accuracy;
    train_sum.precision += train_result.precision;
    train_sum.recall += train_result.recall;
    train_sum.number_svs += train_result.number_svs;
    train_sum.number_bsv += train_result.number_bsv;
    test_sum.loss += test_result.loss;
    test_sum.loss_pos += test_result.loss_pos;
    test_sum.loss_neg += test_result.loss_neg;
    test_sum.MAE += test_result.MAE;
    test_sum.MSE += test_result.MSE;
    test_sum.accuracy += test_result.accuracy;
    test_sum.precision += test_result.precision;
    test_sum.recall += test_result.recall;
    if(verbosity>=4){
      output_stream<<"Training set:"<<endl
	  <<"Loss: "<<train_result.loss<<endl;
      if(parameters->Lpos != parameters->Lneg){
	output_stream<<"  Loss+: "<<train_result.loss_pos<<endl;
	output_stream<<"  Loss-: "<<train_result.loss_neg<<endl;
      };
      output_stream<<"MAE: "<<train_result.MAE<<endl;
      output_stream<<"MSE: "<<train_result.MSE<<endl;
      output_stream<<"VCdim: "<<train_result.VCdim<<endl;
      if(parameters->is_pattern){
	output_stream<<"Accuracy  : "<<train_result.accuracy<<endl
	    <<"Precision : "<<train_result.precision<<endl
	    <<"Recall    : "<<train_result.recall<<endl;
      };
      output_stream<<"Support Vectors : "<<train_result.number_svs<<endl;
      output_stream<<"Bounded SVs     : "<<train_result.number_bsv<<endl;
      output_stream<<"Test set:"<<endl
	  <<"Loss: "<<test_result.loss<<endl;
      if(parameters->Lpos != parameters->Lneg){
	output_stream<<"  Loss+: "<<test_result.loss_pos<<endl;
	output_stream<<"  Loss-: "<<test_result.loss_neg<<endl;
      };
      if(parameters->is_pattern){
	output_stream<<"Accuracy  : "<<test_result.accuracy<<endl
	    <<"Precision : "<<test_result.precision<<endl
	    <<"Recall    : "<<test_result.recall<<endl;
      };
    };
  };
  parameters->verbosity = verbosity;
  number-=parameters->cv_window;
  if(verbosity > 1){
    output_stream<<"----------------------------------------"<<endl;
    output_stream<<"Results of "<<number<<"-fold cross-validation:"<<endl;
    output_stream<<"-- Training set: --"<<endl
	<<"Loss: "<<train_sum.loss/number<<endl;
    if(parameters->Lpos != parameters->Lneg){
      output_stream<<"  Loss+: "<<train_sum.loss_pos/number<<endl;
      output_stream<<"  Loss-: "<<train_sum.loss_neg/number<<endl;
    };
    output_stream<<"MAE: "<<train_sum.MAE/number<<endl;
    output_stream<<"MSE: "<<train_sum.MSE/number<<endl;
    output_stream<<"VCdim: "<<train_sum.VCdim/number<<endl;
    if(parameters->is_pattern){
      output_stream<<"Accuracy  : "<<train_sum.accuracy/number<<endl
	  <<"Precision : "<<train_sum.precision/number<<endl
	  <<"Recall    : "<<train_sum.recall/number<<endl;
    };
    output_stream<<"Support Vectors : "<<((SVMFLOAT)train_sum.number_svs)/number<<endl;
    output_stream<<"Bounded SVs     : "<<((SVMFLOAT)train_sum.number_bsv)/number<<endl;
    output_stream<<"-- Test set: --"<<endl
	<<"Loss: "<<test_sum.loss/number<<endl;
    if(parameters->Lpos != parameters->Lneg){
      output_stream<<"  Loss+: "<<test_sum.loss_pos/number<<endl;
      output_stream<<"  Loss-: "<<test_sum.loss_neg/number<<endl;
    };
    output_stream<<"MAE: "<<test_sum.MAE/number<<endl;
    output_stream<<"MSE: "<<test_sum.MSE/number<<endl;

    if(parameters->is_pattern){
      output_stream<<"Accuracy  : "<<test_sum.accuracy/number<<endl
	  <<"Precision : "<<test_sum.precision/number<<endl
	  <<"Recall    : "<<test_sum.recall/number<<endl;
    };
  };
  test_sum.VCdim = train_sum.VCdim/number;
  test_sum.loss /= number;
  test_sum.loss_pos /= number;
  test_sum.loss_neg /= number;
  test_sum.MAE /= number;
  test_sum.MSE /= number;
  test_sum.pred_loss = train_sum.pred_loss;
  test_sum.accuracy /= number;
  test_sum.precision /= number;
  test_sum.recall /= number;
  test_sum.number_svs = train_sum.number_svs/number;
  test_sum.number_bsv = train_sum.number_bsv/number;
  delete cv_test;
  delete cv_train;

  return test_sum;
};


svm_result train(){
  svm_result the_result;
  if(parameters->cross_validation > 0){
    the_result = do_cv();
  }
  else{
    kernel->init(parameters->kernel_cache,training_set);
    svm->init(kernel,parameters);

    if(parameters->is_nu || parameters->is_distribution){
      output_stream<<"Training started with nu = "
	  <<parameters->nu
	  <<"."<<endl;
    }
    else if(parameters->get_Cpos() == parameters->get_Cneg()){
      output_stream<<"Training started with C = "
	  <<parameters->get_Cpos()
	  <<"."<<endl;
    }
    else{
      output_stream<<"Training started with C = ("<<parameters->get_Cpos()
	  <<","<<parameters->get_Cneg()<<")."<<endl;
    };
    the_result = svm->train(training_set);
  };
  return the_result;
};


inline
SVMFLOAT to_minimize(svm_result result){
  // which value to minimize in calc_c
  if((parameters->cross_validation <= 0) && (1 == parameters->is_pattern)){
    return (1.0-result.pred_accuracy);
  }
  else{
    if(1 == parameters->is_pattern){
      return (1.0-result.accuracy);
    }
    else{
      return result.loss;
    };
  };
};

svm_result calc_c(){
  const SVMFLOAT lambda = 0.618033989; // (sqrt(5)-1)/2
  SVMINT verbosity = parameters->verbosity;
  parameters->verbosity -= 2;
  svm_result the_result;
  SVMFLOAT c_min = parameters->c_min;
  SVMFLOAT c_max = parameters->c_max;
  SVMFLOAT c_delta = parameters->c_delta;
  SVMFLOAT oldC;
  SVMINT last_dec=0; // when did loss decrease?
  // setup s,t
  if(verbosity >= 3){
    output_stream<<"starting search for C"<<endl;
  };
  if((parameters->search_c == 'a') ||(parameters->search_c == 'm')){
    SVMFLOAT minimal_value=infinity;
    SVMFLOAT minimal_C=c_min;
    svm_result minimal_result;
    SVMFLOAT result_value;
    oldC=c_min;
    training_set->clear_alpha();
    while(c_min <= c_max){
      if(verbosity>=3){
	output_stream<<"C = "<<c_min<<" :"<<endl;
      };
      parameters->realC = c_min;
      training_set->scale_alphas(c_min/oldC);
      // training_set->clear_alpha();
      oldC = c_min;
      if(verbosity >= 4){
	output_stream<<"C = "<<c_min<<endl;
      }
      the_result = train();
      if(verbosity>=3){
	output_stream<<"loss = "<<the_result.loss<<endl;
	if(parameters->is_pattern){
	  output_stream<<"accuracy = "<<the_result.accuracy<<endl;
	  // output_stream<<"predicted loss = "<<the_result.pred_loss<<endl;
	  output_stream<<"predicted accuracy = "<<the_result.pred_accuracy<<endl;
	};
	//	output_stream<<"VCdim <= "<<the_result.VCdim<<endl;
      };
      result_value = to_minimize(the_result);
      //      output_stream<<result_value<<endl;
      last_dec++;
      if((result_value<minimal_value) && (! isnan(result_value))){
	minimal_value=result_value;
	minimal_C=c_min;
	minimal_result = the_result;
	last_dec=0;
      };
      if(parameters->search_c == 'a'){
	c_min += c_delta;
      }
      else{
	c_min *= c_delta;
      };
      if((parameters->search_stop > 0) && (last_dec >= parameters->search_stop)){
	// no decrease in loss, stop
	c_min = 2*c_max;
      };
    };
    parameters->realC = minimal_C;
    the_result = minimal_result;
  }
  else{
    // method of golden ratio

    SVMFLOAT s = lambda*c_min+(1-lambda)*c_max;
    SVMFLOAT t = (1-lambda)*c_min+lambda*c_max;
    SVMFLOAT phi_s;
    SVMFLOAT phi_t;

    parameters->realC = s;
    training_set->clear_alpha();
    the_result = train();
    phi_s = to_minimize(the_result);
    parameters->realC = t;
    training_set->scale_alphas(t/s);
    oldC = t;
    the_result = train();
    phi_t = to_minimize(the_result);
    while(c_max - c_min > c_delta*c_min){
      if(verbosity >= 3){
	output_stream<<"C in ["<<c_min<<","<<c_max<<"]"<<endl;
      };
      if(phi_s < phi_t){
	c_max = t;
	t = s;
	phi_t = phi_s;
	// calc s
	s = lambda*c_min+(1-lambda)*c_max;
	parameters->realC = s;
	training_set->scale_alphas(s/oldC);
	oldC=s;
	the_result = train();
	phi_s = to_minimize(the_result);
      }
      else{
	c_min = s;
	s = t;
	phi_s = phi_t;
	// calc t
	t = (1-lambda)*c_min+lambda*c_max;
	parameters->realC = t;
	training_set->scale_alphas(t/oldC);
	oldC=t;
	the_result = train();
	phi_t = to_minimize(the_result);
      };
    };
    // save last results
    if(phi_s < phi_t){
      c_max = t;
    }
    else{
      c_min = s;
    };
    parameters->realC = (c_min+c_max)/2;

  };

  // ouput result
  if(verbosity >= 1){
    output_stream<<"*** Optimal C is "<<parameters->realC;
    if(parameters->search_c == 'g'){
      output_stream<<" +/-"<<((c_max-c_min)/2);
    };
    output_stream<<endl;
  };
  if(verbosity>=2){
    output_stream<<"result:"<<endl
	<<"Loss: "<<the_result.loss<<endl;
    if(parameters->Lpos != parameters->Lneg){
      output_stream<<"  Loss+: "<<the_result.loss_pos<<endl;
      output_stream<<"  Loss-: "<<the_result.loss_neg<<endl;
    };
    if(parameters->is_pattern){
      output_stream<<"predicted Loss: "<<the_result.pred_loss<<endl;
    };
    output_stream<<"MAE: "<<the_result.MAE<<endl;
    output_stream<<"MSE: "<<the_result.MSE<<endl;
    output_stream<<"VCdim <= "<<the_result.VCdim<<endl;

    if(parameters->is_pattern){
      output_stream<<"Accuracy  : "<<the_result.accuracy<<endl
	  <<"Precision : "<<the_result.precision<<endl
	  <<"Recall    : "<<the_result.recall<<endl;
      if(parameters->cross_validation == 0){
	output_stream<<"predicted Accuracy  : "<<the_result.pred_accuracy<<endl
	    <<"predicted Precision : "<<the_result.pred_precision<<endl
	    <<"predicted Recall    : "<<the_result.pred_recall<<endl;
      };
    };
    output_stream<<"Support Vectors : "<<the_result.number_svs<<endl;
    output_stream<<"Bounded SVs     : "<<the_result.number_bsv<<endl;
    if(parameters->search_c == 'g'){
      output_stream<<"(WARNING: this is the last result attained and may slightly differ from the result of the optimal C!)"<<endl;
    };
  };
  parameters->verbosity = verbosity;
  return the_result;
};






int learn() {
//mvs

kernel=0;
parameters=0;
training_set=0;
is_linear=1;
test_sets = 0;

    output_stream.open (output_filename, fstream::trunc);
    if(!output_stream) {
        MessageBox(NULL, output_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }


ofstream test_stream;
fstream kernel_stream;
ifstream kernel_stream2;

    test_stream.open(kernel_filename, fstream::trunc);
    if(!test_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    test_stream.close();





  if(!InputFilename[0])
            MessageBox(NULL, "No training file selected", "Error", MB_OK | MB_ICONINFORMATION);
  else {



//Martin's hack
int iLength;
PTSTR pstrDoc;
    char OutputFilenamesvm[256];
    char OutputFilenamepred[256];


if (SendMessage(hwndEditPattern,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_pattern = true;
else
    windows_is_pattern = false;

if (SendMessage(hwndEditRegression,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_regression = true;
else
    windows_is_regression = false;



//C
iLength = GetWindowTextLength (hwndEditC);
if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
    return false;
GetWindowText (hwndEditC, pstrDoc, iLength + 1) ;
  if(iLength > 0)
      windows_is_C = true;
  else
      windows_is_C = false;
windows_C = atof(pstrDoc);
free(pstrDoc);



//Epsilon
if (SendMessage(hwndEditE,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_epsilon = true;
else
    windows_is_epsilon = false;

    iLength = GetWindowTextLength (hwndEditE1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditE1, pstrDoc, iLength + 1) ;
    windows_epsilon = atof(pstrDoc);
    free(pstrDoc);



//Nu

if (SendMessage(hwndEditNu,BM_GETCHECK,0,0)==BST_CHECKED)
    windows_is_nu = true;
else
    windows_is_nu = false;


    iLength = GetWindowTextLength (hwndEditNu1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditNu1, pstrDoc, iLength + 1) ;
    windows_nu = atof(pstrDoc);
    free(pstrDoc);









    //Dot
if (SendMessage(hwndEditD,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type dot" << endl;
    kernel_stream.close();
}



    //Polynomial
if (SendMessage(hwndEditP,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type polynomial" << endl;
    iLength = GetWindowTextLength (hwndEditP1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditP1, pstrDoc, iLength + 1) ;
    kernel_stream << "degree " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}




//Radial
if (SendMessage(hwndEditR,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type radial" << endl;
    iLength = GetWindowTextLength (hwndEditR1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditR1, pstrDoc, iLength + 1) ;
    kernel_stream << "gamma " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}



//Neural
if (SendMessage(hwndEditN,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type neural" << endl;
    iLength = GetWindowTextLength (hwndEditN1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditN1, pstrDoc, iLength + 1) ;
    kernel_stream << "a " << pstrDoc << endl;
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditN2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditN2, pstrDoc, iLength + 1) ;
    kernel_stream << "b " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}


//Anova
if (SendMessage(hwndEditA,BM_GETCHECK,0,0)==BST_CHECKED) {
    kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
    if(!kernel_stream) {
        MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
    kernel_stream << "type anova" << endl;
    iLength = GetWindowTextLength (hwndEditA1);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditA1, pstrDoc, iLength + 1) ;
    kernel_stream << "gamma " << pstrDoc << endl;
    free(pstrDoc);
    iLength = GetWindowTextLength (hwndEditA2);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        return false;
    GetWindowText (hwndEditA2, pstrDoc, iLength + 1) ;
    kernel_stream << "degree " << pstrDoc << endl;
    free(pstrDoc);
    kernel_stream.close();
}




  output_stream<<"*** mySVM version "<<mysvmversion<<" ***"<<endl;
  output_stream.precision(8);
  // read objects


    try{


	    output_stream<<"Reading "<<InputFilename<<endl;
	    //ifstream input_file(InputFilename);
	    input_file.open (InputFilename, fstream::in);



     if(input_file.bad()) {
        MessageBox(NULL, InputFilename, "ERROR: Could not read file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }
	    //read_input(input_file,InputFilename);
	    read_input();
	    input_file.close();


  }


       catch(general_exception &the_ex) {
        MessageBox(NULL, the_ex.error_msg, "*** Error while reading input: ", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }





  catch(...){
  MessageBox(NULL, "Error", "*** Program ended because of unknown error while reading input", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }


  if(0 == parameters){
    parameters = new parameters_c();
    if(training_set->initialised_pattern_y()){
      parameters->is_pattern = 1;
      parameters->do_scale_y = 0;
    };
  };
  parameters->is_linear = is_linear;
  if(0 == kernel){
    kernel = new kernel_dot_c();
  };
  if(0 == training_set){
        MessageBox(NULL, "Error", "*** ERROR: You did not enter the training set", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    };

  if(2 > training_set->size()){
          MessageBox(NULL, "Error", "*** ERROR: Need at least two examples to learn.", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    };

if(parameters->is_distribution){
    svm = new svm_distribution_c();
    output_stream<<"distribution estimation SVM generated"<<endl;
  }
  else if(parameters->is_nu){
    if(parameters->is_pattern){
      svm = new svm_nu_pattern_c();
      if(! training_set->initialised_pattern_y()){
	output_stream<<"WARNING: Parameters set a pattern SVM, but the training ys are not in {-1,1}."<<endl;
      }
      output_stream<<"nu-PSVM generated"<<endl;
    }
    else{
      svm = new svm_nu_regression_c();
      output_stream<<"nu-RSVM generated"<<endl;
    };
  }
  else if(parameters->is_pattern){
    svm = new svm_pattern_c();
    if(! training_set->initialised_pattern_y()){
      output_stream<<"WARNING: Parameters set a pattern SVM, but the training ys are not in {-1,1}."<<endl;
    }
    output_stream<<"PSVM generated"<<endl;
  }
  else{
    svm = new svm_regression_c();
    output_stream<<"RSVM generated"<<endl;
  };

  // scale examples
  if(parameters->do_scale){
    training_set->scale(parameters->do_scale_y);
  };

  // training the svm
  if(parameters->search_c != 'n'){
    calc_c();
    output_stream<<"re-training without CV and C = "<<parameters->realC<<endl;
    parameters->cross_validation = 0;
    parameters->verbosity -= 1;
    train();
    parameters->verbosity += 1;
  }
  else{
    train();
  };

    int length = strlen(InputFilename);

    strcpy(OutputFilenamesvm, InputFilename);
    OutputFilenamesvm[length - 4] = '-';
    OutputFilenamesvm[length - 3] = 's';
    OutputFilenamesvm[length - 2] = 'v';
    OutputFilenamesvm[length - 1] = 'm';
    OutputFilenamesvm[length + 0] = InputFilename[length -4];
    OutputFilenamesvm[length + 1] = InputFilename[length -3];
    OutputFilenamesvm[length + 2] = InputFilename[length -2];
    OutputFilenamesvm[length + 3] = InputFilename[length -1];
    OutputFilenamesvm[length + 4] = '\0';

    strcpy(OutputFilenamepred, InputFilename);
    OutputFilenamepred[length - 4] = '-';
    OutputFilenamepred[length - 3] = 'p';
    OutputFilenamepred[length - 2] = 'r';
    OutputFilenamepred[length - 1] = 'e';
    OutputFilenamepred[length + 0] = 'd';
    OutputFilenamepred[length + 1] = InputFilename[length -4];
    OutputFilenamepred[length + 2] = InputFilename[length -3];
    OutputFilenamepred[length + 3] = InputFilename[length -2];
    OutputFilenamepred[length + 4] = InputFilename[length -1];
    OutputFilenamepred[length + 5] = '\0';

  if(0 == parameters->cross_validation){
    // save results
    if(parameters->verbosity > 1)
        output_stream<<"Saving trained SVM to "<<OutputFilenamesvm<<endl;
    ofstream output_file(OutputFilenamesvm,ios::out|ios::trunc);
    output_file.precision(16);
    output_file<<*training_set;
    output_file.close();
    //delete []outname;
  };

  // testing
  if((parameters->cross_validation > 0) && (0 != test_sets)){
    // test result of cross validation: train new SVM on whole example set
    parameters->cross_validation = 0;
    output_stream<<"Re-training SVM on whole example set for testing"<<endl;
    train();
  };
  if(0 != test_sets){
    output_stream<<"----------------------------------------"<<endl;
    output_stream<<"Starting tests"<<endl;
    example_set_c* next_test;
    SVMINT test_no = 0;
    char* outname = new char[MAXCHAR];
    while(test_sets != 0){
      test_no++;
      next_test = test_sets->the_set;
      if(parameters->do_scale){
	next_test->scale(training_set->get_exp(),
			 training_set->get_var(),
			 training_set->get_dim());
      };
      if(next_test->initialised_y()){
	output_stream<<"Testing examples from file "<<(next_test->get_filename())<<endl;
	svm->test(next_test,1);
      }
      else{
	output_stream<<"Predicting examples from file "<<(next_test->get_filename())<<endl;
	svm->predict(next_test);
	// output to file .pred

	//strcpy(outname,next_test->get_filename());
	//strcat(outname,".pred");
	//ofstream output_file(outname,ios::out);
	ofstream output_file(OutputFilenamepred,ios::out);
	output_file<<"@examples"<<endl;
	output_file<<(*next_test);
	output_file.close();
      };
      test_sets = test_sets->next; // skip delete!
    };
    delete []outname;
  };
  if(kernel) delete kernel;
  delete svm;
  if(parameters->verbosity > 1){
    output_stream << "mysvm ended successfully."<<endl;
  };
    MessageBox(NULL, "Finished", "winSVM", MB_OK | MB_ICONINFORMATION);
}
output_stream.close();
  return 0;
};




//mvs
int optimize() {
    int iLength;
    PTSTR pstrDoc;
    int windows_runs;
    fstream kernel_stream;
    ifstream kernel_stream2;
    ofstream optimize_stream;

    char OptimizeFilename[256];
    char OutputFilenamesvm[256];
    char OutputFilenamepred[256];

    output_stream.open (output_filename, fstream::trunc);
    if(!output_stream) {
        MessageBox(NULL, output_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
        exit(-1);
    }

    int length = strlen(InputFilename);
    strcpy(OptimizeFilename, InputFilename);
    OptimizeFilename[length - 4] = '-';
    OptimizeFilename[length - 3] = 'o';
    OptimizeFilename[length - 2] = 'p';
    OptimizeFilename[length - 1] = 't';
    OptimizeFilename[length + 0] = '.';
    OptimizeFilename[length + 1] = 'c';
    OptimizeFilename[length + 2] = 's';
    OptimizeFilename[length + 3] = 'v';
    OptimizeFilename[length + 4] = '\0';




    optimize_stream.open(OptimizeFilename, fstream::out);
    if(!optimize_stream)
        MessageBox(NULL, OptimizeFilename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
    else
    {

    iLength = GetWindowTextLength (hwndEditRuns);
    if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR))))
        exit(0);
    GetWindowText (hwndEditRuns, pstrDoc, iLength + 1) ;
    if(iLength > 0) {
        windows_runs = atoi(pstrDoc);
    }
    else windows_runs = 0;
    free(pstrDoc);
    
    
    
    float cwr;
    int ti;
    int ui;
    char tc;
    char uc;
    int poly_degree;
    float radial_gamma;
    float a;
    float b;
    float anova_gamma;
    float anova_degree;
    int ent;
    int nut;
    
    int temp_radial_gamma;

    optimize_stream << "MSE,C,epsilon,nu,kernel" << endl;


    perc[0] = ' ';
    perc[1] = ' ';
    perc[2] = '0';
    perc[3] = '%';
    perc[4] = '\0';
    SendMessage(hwndEditCounter, WM_SETTEXT, 0, (LPARAM)perc);

    for (int count = 1; count <= windows_runs; count++) {
        kernel=0;
        parameters=0;
        training_set=0;
        is_linear=1;
        test_sets = 0;

        if (SendMessage(hwndEditPattern,BM_GETCHECK,0,0)==BST_CHECKED)
            windows_is_pattern = true;
        else
            windows_is_pattern = false;
            
        if (SendMessage(hwndEditRegression,BM_GETCHECK,0,0)==BST_CHECKED)
            windows_is_regression = true;
        else
            windows_is_regression = false;
            
        if(!InputFilename[0])
            MessageBox(NULL, "No training file selected", "Error", MB_OK | MB_ICONINFORMATION);
        else {
            //C            
            
       	    int ct = random_range(0,8);
                	    switch (ct) {
                	        case 0:
                	            windows_C = 0.01;
            	            break;                               
                	        case 1:
                	            windows_C = 0.1;
            	            break;
                	        case 2:
                	            windows_C = 1;
            	            break;
                	        case 3:
                	            windows_C = 10;
            	            break;
                	        case 4:
                	            windows_C = 100;
            	            break;
                	        case 5:
                	            windows_C = 1000;
            	            break;
                	        case 6:
                	            windows_C = 10000;
            	            break;
                	        case 7:
                	            windows_C = 100000;
            	            break;
                	        case 8:
                	            windows_C = 1000000;
            	            break;
        	        }    
            
            
            

            
            output_stream<<endl<<"C "<<windows_C<<endl;
            if(!windows_C)
                MessageBox(NULL, "No C entered", "Error", MB_OK | MB_ICONINFORMATION);
            else {

                //epsilon or nu
                int en = random_range(0,1);
                switch (en) {
                	case 0:
                	    windows_is_epsilon = true;
                	    windows_is_nu = false;
                	    
                	    ent = random_range(0,6);
                	    switch (ent) {
                	        case 0:
                	            windows_epsilon = 0.000001;
            	            break;
                	        case 1:
                	            windows_epsilon = 0.00001;
            	            break;
                	        case 2:
                	            windows_epsilon = 0.0001;
            	            break;
                	        case 3:
                	            windows_epsilon = 0.001;
            	            break;
                	        case 4:
                	            windows_epsilon = 0.01;
            	            break;
                	        case 5:
                	            windows_epsilon = 0.1;
            	            break;
                	        case 6:
                	            windows_epsilon = 1;
            	            break;
        	            }    
        	            output_stream<<"epsilon "<<windows_epsilon<<endl;
            	    break;
            	    
           	    	case 1:
           	    	    windows_is_nu = true;
           	    	    windows_is_epsilon = false;

                	    nut = random_range(0,5);
                	    switch (nut) {
                	        case 0:
                	            windows_nu = 0;
            	            break;
                	        case 1:
                	            windows_nu = 0.2;
            	            break;
                	        case 2:
                	            windows_nu = 0.4;
            	            break;
                	        case 3:
                	            windows_nu = 0.6;
            	            break;
                	        case 4:
                	            windows_nu = 0.8;
            	            break;
                	        case 5:
                	            windows_nu = 1;
            	            break;
            	        }    
           	    	    output_stream<<"nu "<<windows_nu<<endl;
           	    	    break;
                     }

                 //Choose kernel at random
                 int kern = random_range(1,5);
                 switch (kern) {
                     case 1:
                         //Dot
                         kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
                         if(!kernel_stream) {
                             MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
                             exit(-1);
                         }
                         kernel_stream << "type dot" << endl;
                         output_stream << "type dot" << endl;
                         kernel_stream.close();
                     break;

                     case 2:
                         //Polynomial
                         kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
                         if(!kernel_stream) {
                             MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
                             exit(-1);
                         }
                         poly_degree = random_range(2,6);
                         kernel_stream << "type polynomial" << endl;
                         kernel_stream << "degree " << poly_degree << endl;
                         output_stream << "type polynomial" << endl;
                         output_stream << "degree " << poly_degree << endl;
                         kernel_stream.close();
                     break;


                     case 3:
                         //Radial
                         kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
                         if(!kernel_stream) {
                             MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
                             exit(-1);
                         }
                         temp_radial_gamma = random_range(1,19);
                         if (temp_radial_gamma == 16)
                             radial_gamma = 0.001;
                         if (temp_radial_gamma == 17)
                             radial_gamma = 0.01;
                         if (temp_radial_gamma == 18)
                             radial_gamma = 0.05;                         
                         if (temp_radial_gamma <= 10)
                             radial_gamma = 0.1 * (float) temp_radial_gamma;
                         if (temp_radial_gamma == 11)
                             radial_gamma = 1.5;
                         if (temp_radial_gamma == 12)
                             radial_gamma = 2;
                         if (temp_radial_gamma == 13)
                             radial_gamma = 5;
                         if (temp_radial_gamma == 14)
                             radial_gamma = 10;
                         if (temp_radial_gamma == 15)
                             radial_gamma = 100;                         
                         if (temp_radial_gamma == 19)
                             radial_gamma = 1000; 

                         
                         kernel_stream << "type radial" << endl;
                         kernel_stream << "gamma " << radial_gamma << endl;
                         output_stream << "type radial" << endl;
                         output_stream << "gamma " << radial_gamma << endl;
                         kernel_stream.close();
                     break;


                     case 4:
                         //Neural
                         kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
                         if(!kernel_stream) {
                             MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
                             exit(-1);
                         }
                         a = (float) random_range(0,10)/10;
                         b = (float) random_range(0,10)/10;
                         kernel_stream << "type neural" << endl;
                         kernel_stream << "a " << a << endl;
                         kernel_stream << "b " << b << endl;
                         output_stream << "type neural" << endl;
                         output_stream << "a " << a << endl;
                         output_stream << "b " << b << endl;
                         kernel_stream.close();
                     break;

                     case 5:
                         //Anova
                         kernel_stream.open(kernel_filename, fstream::out | fstream::trunc);
                         if(!kernel_stream) {
                             MessageBox(NULL, kernel_filename, "Failed to open file", MB_OK | MB_ICONINFORMATION);
                             exit(-1);
                         }
                         anova_gamma = 0.1 * (float) random_range(0,10);
                         anova_degree = random_range(1,5);
                         kernel_stream << "type anova" << endl;
                         kernel_stream << "gamma " << anova_gamma << endl;
                         kernel_stream << "degree " << anova_degree << endl;
                         output_stream << "type anova" << endl;
                         output_stream << "gamma " << anova_gamma << endl;
                         output_stream << "degree " << anova_degree << endl;
                         kernel_stream.close();
                     break;
                }
                
                output_stream<<"*** mySVM version "<<mysvmversion<<" ***"<<endl;
                output_stream.precision(8);
                // read objects

                try {
                    output_stream<<"Reading "<<InputFilename<<endl;
                    input_file.open (InputFilename, fstream::in);
                    if(input_file.bad()) {
                        MessageBox(NULL, InputFilename, "ERROR: Could not read file", MB_OK | MB_ICONINFORMATION);
                        exit(-1);
                    }
                    read_input();
                    input_file.close();
                }
                catch(general_exception &the_ex) {
                    MessageBox(NULL, the_ex.error_msg, "*** Error while reading input: ", MB_OK | MB_ICONINFORMATION);
                    exit(-1);
                }
                catch(...){
                    MessageBox(NULL, "Error", "*** Program ended because of unknown error while reading input", MB_OK | MB_ICONINFORMATION);
                    exit(-1);
                }
                
                if(0 == parameters) {
                    parameters = new parameters_c();
                    if(training_set->initialised_pattern_y()){
                        parameters->is_pattern = 1;
                        parameters->do_scale_y = 0;
                    };
                };
                parameters->is_linear = is_linear;
                if(0 == kernel) {
                    kernel = new kernel_dot_c();
                };
                if(0 == training_set) {
                    MessageBox(NULL, "Error", "*** ERROR: You did not enter the training set", MB_OK | MB_ICONINFORMATION);
                    exit(-1);
                };
                if(2 > training_set->size()) {
                    MessageBox(NULL, "Error", "*** ERROR: Need at least two examples to learn.", MB_OK | MB_ICONINFORMATION);
                    exit(-1);
                };
                if(parameters->is_distribution) {
                    svm = new svm_distribution_c();
                    output_stream<<"distribution estimation SVM generated"<<endl;
                }
                else if(parameters->is_nu) {
                    if(parameters->is_pattern){
                        svm = new svm_nu_pattern_c();
                        if(! training_set->initialised_pattern_y()){
                            output_stream<<"WARNING: Parameters set a pattern SVM, but the training ys are not in {-1,1}."<<endl;
                        }
                        output_stream<<"nu-PSVM generated"<<endl;
                    }
                    else {
                        svm = new svm_nu_regression_c();
                        output_stream<<"nu-RSVM generated"<<endl;
                    };
                }
                else if(parameters->is_pattern) {
                    svm = new svm_pattern_c();
                    if(! training_set->initialised_pattern_y()) {
                        output_stream<<"WARNING: Parameters set a pattern SVM, but the training ys are not in {-1,1}."<<endl;
                    }
                    output_stream<<"PSVM generated"<<endl;
                }
                else {
                    svm = new svm_regression_c();
                    output_stream<<"RSVM generated"<<endl;
                };

                // scale examples
                if(parameters->do_scale){
                    training_set->scale(parameters->do_scale_y);
                };

                // training the svm
                if(parameters->search_c != 'n'){
                    calc_c();
                    output_stream<<"re-training without CV and C = "<<parameters->realC<<endl;
                    parameters->cross_validation = 0;
                    parameters->verbosity -= 1;
                    train();
                    parameters->verbosity += 1;
                }
                else {
                    train();
                };
                
                if(0 == parameters->cross_validation) {
                    // save results
                    if(parameters->verbosity > 1){
                        output_stream<<"Saving trained SVM to "<<(training_set->get_filename())<<".svm"<<endl;
                    };
                    int length = strlen(InputFilename);
                    strcpy(OutputFilenamesvm, InputFilename);
                    OutputFilenamesvm[length - 4] = '-';
                    OutputFilenamesvm[length - 3] = 's';
                    OutputFilenamesvm[length - 2] = 'v';
                    OutputFilenamesvm[length - 1] = 'm';
                    OutputFilenamesvm[length + 0] = InputFilename[length -4];
                    OutputFilenamesvm[length + 1] = InputFilename[length -3];
                    OutputFilenamesvm[length + 2] = InputFilename[length -2];
                    OutputFilenamesvm[length + 3] = InputFilename[length -1];
                    OutputFilenamesvm[length + 4] = '\0';
                    
                    strcpy(OutputFilenamepred, InputFilename);
                    OutputFilenamepred[length - 4] = '-';
                    OutputFilenamepred[length - 3] = 'p';
                    OutputFilenamepred[length - 2] = 'r';
                    OutputFilenamepred[length - 1] = 'e';
                    OutputFilenamepred[length + 0] = 'd';
                    OutputFilenamepred[length + 1] = InputFilename[length -4];
                    OutputFilenamepred[length + 2] = InputFilename[length -3];
                    OutputFilenamepred[length + 3] = InputFilename[length -2];
                    OutputFilenamepred[length + 4] = InputFilename[length -1];
                    OutputFilenamepred[length + 5] = '\0';
                    
                    ofstream output_file(OutputFilenamesvm,ios::out|ios::trunc);
                    output_file.precision(16);
                    output_file<<*training_set;
                    output_file.close();
                    //delete []outname;
                };

                // testing
                if((parameters->cross_validation > 0) && (0 != test_sets)) {
                    // test result of cross validation: train new SVM on whole example set
                    parameters->cross_validation = 0;
                    output_stream<<"Re-training SVM on whole example set for testing"<<endl;
                    train();
                };
                
                if(0 != test_sets) {
                    output_stream<<"----------------------------------------"<<endl;
                    output_stream<<"Starting tests"<<endl;
                    example_set_c* next_test;
                    SVMINT test_no = 0;
                    char* outname = new char[MAXCHAR];
                    while(test_sets != 0){
                    test_no++;
                    next_test = test_sets->the_set;
                    if(parameters->do_scale){
                        next_test->scale(training_set->get_exp(),
                        training_set->get_var(),
                        training_set->get_dim());
                    };
                    if(next_test->initialised_y()) {
                        output_stream<<"Testing examples from file "<<(next_test->get_filename())<<endl;
                        svm->test(next_test,1);
                    }
                    else {
	                    output_stream<<"Predicting examples from file "<<(next_test->get_filename())<<endl;
	                    svm->predict(next_test);
	                    // output to file .pred
	                    ofstream output_file(OutputFilenamepred,ios::out);
	                    output_file<<"@examples"<<endl;
	                    output_file<<(*next_test);
	                    output_file.close();
                    };
                    test_sets = test_sets->next; // skip delete!
                };
                delete []outname;
            };
            if (kernel)
                delete kernel;
            delete svm;
            if (parameters->verbosity > 1)
                output_stream << "mysvm ended successfully."<<endl;
                
            //Martin's hack
            if (windows_is_epsilon)
                optimize_stream << global_MSE << "," << windows_C << "," <<windows_epsilon << "," << " " << "," << k[0] << "," << k[1] << "," << k[2] << endl;
            if (windows_is_nu)
                optimize_stream << global_MSE << "," << windows_C << "," << " " << "," << windows_nu << "," << k[0] << "," << k[1] << "," << k[2] << endl;

            //  MessageBox(NULL, "Finished", "winSVM", MB_OK | MB_ICONINFORMATION);
            //  return 0;

            cwr = (float)((float)count/(float)windows_runs);
            cwr = (float)100*cwr;

            ti = (int) (cwr/10);
            ui = (int) cwr % 10;
            tc = (char) (ti + '0');
            uc = (char) (ui + '0');
            perc[0] = ' ';
            perc[1] = tc;
            if (perc[1] == '0')
                perc[1] = ' ';
            perc[2] = uc;
            perc[3] = '%';
            perc[4] = '\0';
            
            SendMessage(hwndEditCounter, WM_SETTEXT, 0, (LPARAM)perc);
            }
        }
    }
    perc[0] = '1';
    perc[1] = '0';
    perc[2] = '0';
    perc[3] = '%';
    perc[4] = '\0';
    SendMessage(hwndEditCounter, WM_SETTEXT, 0, (LPARAM)perc);
    
}    
    
    output_stream.close();
    return 0;
}






int GetOpenFilename(char *MatchType, char *MatchFile)
{
   OPENFILENAME fn;
   register int i;
   char Filter[128];
   // construct file name filter string
   lstrcpy(Filter,MatchType);
   i = lstrlen(Filter) + 1;
   lstrcpy(&Filter[i],MatchFile);
   i += lstrlen(&Filter[i]) + 1;
   Filter[i] = '\0';
   // fill in structure fields for Open File dialog box
   fn.lStructSize       = sizeof(OPENFILENAME);
   fn.hwndOwner         = GetActiveWindow();
   fn.lpstrFilter       = Filter;
   fn.lpstrCustomFilter = NULL;
   fn.nFilterIndex      = 0;
   fn.lpstrFile         = NULL;
   fn.lpstrFile         = DefaultInputFile;
   fn.nMaxFile          = sizeof(DefaultInputFile);
   fn.lpstrFileTitle    = NULL;
   fn.lpstrInitialDir   = NULL;
   fn.lpstrTitle        = NULL;
   fn.Flags             = OFN_READONLY;
   fn.lpstrDefExt       = &MatchFile[2];
   *DefaultInputFile = '\0';
   // activate the Open File dialog box
   if (GetOpenFileName(&fn))
   {
      lstrcpy(InputFilename,DefaultInputFile);
      return 1;
   }
   else
      return -1;
}





/*  Declare Windows procedure  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);

/*  Make the class name into a global variable  */
char szClassName[ ] = "winSVM";

int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)

{

    HWND hwnd;               /* This is the handle for our window */
    MSG messages;            /* Here messages to the application are saved */
    WNDCLASSEX wincl;        /* Data structure for the windowclass */

    /* The Window structure */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = CS_DBLCLKS;                 /* Catch double-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = szClassName;                 /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use Windows's default color as the background of the window */
    //wincl.hbrBackground = (HBRUSH) COLOR_BACKGROUND;
    //wincl.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
    wincl.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);



//Martin's hack
srand(static_cast<unsigned>(time(0)));


    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (
           0,                   /* Extended possibilites for variation */
           szClassName,         /* Classname */
           "winSVM",            /* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           440,                 /* The programs width 544 */
           525,                 /* and height in pixels 375 */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           hThisInstance,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    CheckRadioButton (hwnd, IDB_BUTTON_PATTERN, IDB_BUTTON_REGRESSION, IDB_BUTTON_REGRESSION);
    CheckRadioButton (hwnd, IDB_BUTTON_E, IDB_BUTTON_NU, IDB_BUTTON_E);
    CheckRadioButton (hwnd, IDB_BUTTON_D, IDB_BUTTON_A, IDB_BUTTON_D);

    /* Make the window visible on the screen */
    ShowWindow (hwnd, nFunsterStil);

    /* Run the message loop. It will run until GetMessage() returns 0 */
    while (GetMessage (&messages, NULL, 0, 0))
    {
        if(!IsDialogMessage(hwnd, &messages) ){
        /* Translate virtual-key messages into character messages */
        TranslateMessage(&messages);
        /* Send message to WindowProcedure */
        DispatchMessage(&messages);
        }    
    }

    /* The program return-value is 0 - The value that PostQuitMessage() gave */
    return messages.wParam;
}


/*  This function is called by the Windows function DispatchMessage()  */

LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    HMENU hMenu;
    hMenu = GetMenu (hwnd) ;
    HDC hdc ;
    PAINTSTRUCT ps ;
    TEXTMETRIC tm ;
    HINSTANCE hInstance = (HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE);




    switch (message)                  /* handle the messages */
    {
    case WM_CREATE:



            InvalidateRect(hwnd, NULL, TRUE);



            //Input
            hWndButtonI = CreateWindowEx(
                0,
                "BUTTON",
                "Input file...",
                WS_VISIBLE | WS_CHILD | WS_TABSTOP,
                20,
                20,
                90,
                20,
                hwnd,
                (HMENU) IDB_BUTTON_INPUT,
                hInstance,
                NULL);



            hwndEditInputFilename = CreateWindowEx (WS_EX_CLIENTEDGE, TEXT ("static"), InputFilename,
                         WS_CHILD | WS_VISIBLE | ES_READONLY,
                         140, 20, 265, 20,
                         hwnd, (HMENU) ID_EDIT_INPUT, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;



  CreateWindow(TEXT("button"), TEXT(""), WS_CHILD|WS_VISIBLE|BS_GROUPBOX,10,45,120,60,hwnd,(HMENU)0,hInstance,NULL);



            //pattern
            hwndEditPattern = CreateWindow (TEXT ("button"), "Pattern",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP | WS_TABSTOP,
                         20, 60, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_PATTERN, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;


            //regression
            hwndEditRegression = CreateWindow (TEXT ("button"), "Regression",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 80, 100, 20,
                         hwnd, (HMENU) IDB_BUTTON_REGRESSION, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;



  CreateWindow(TEXT("button"), TEXT(""), 
        WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
        10,115,180,95,hwnd,(HMENU)0,hInstance,NULL);
            //C
            hwndEditC = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         100, 130, 70, 20,
                         hwnd, (HMENU) ID_EDIT_C, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;

            //epsilon
            hwndEditE = CreateWindow (TEXT ("button"), "epsilon",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | BST_CHECKED | WS_GROUP | WS_TABSTOP,
                         20, 160, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_E, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            //nu
            hwndEditNu = CreateWindow (TEXT ("button"), "nu",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 180, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_NU, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;                         
                         
                         
            //epsilon             
            hwndEditE1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         100, 160, 70, 20,
                         hwnd, (HMENU) ID_EDIT_E, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;

            //nu

            hwndEditNu1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         100, 180, 70, 20,
                         hwnd, (HMENU) ID_EDIT_NU, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;


  CreateWindow(TEXT("button"), TEXT("Kernel"), WS_CHILD|WS_VISIBLE|BS_GROUPBOX,10,220,410,130,hwnd,(HMENU)0,hInstance,NULL);


//radio
            hwndEditD = CreateWindow (TEXT ("button"), "dot",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP | WS_TABSTOP,
                         20, 240, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_D, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditP = CreateWindow (TEXT ("button"), "polynomial",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 260, 100, 20,
                         hwnd, (HMENU) IDB_BUTTON_P, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditR = CreateWindow (TEXT ("button"), "radial",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 280, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_R, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditN = CreateWindow (TEXT ("button"), "neural",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 300, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_N, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditA = CreateWindow (TEXT ("button"), "anova",
                         WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON,
                         20, 320, 70, 20,
                         hwnd, (HMENU) IDB_BUTTON_A, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;



            //dot


            //polynomial

            hwndEditP1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | ES_NUMBER | WS_TABSTOP,
                         195, 260, 70, 20,
                         hwnd, (HMENU) ID_EDIT_P, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;


            //radial

            hwndEditR1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         195, 280, 70, 20,
                         hwnd, (HMENU) ID_EDIT_R, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;


            //neural

            hwndEditN1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         195, 300, 70, 20,
                         hwnd, (HMENU) ID_EDIT_N_1, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditN2 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         335, 300, 70, 20,
                         hwnd, (HMENU) ID_EDIT_N_2, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;

            //anova

            hwndEditA1 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         195, 320, 70, 20,
                         hwnd, (HMENU) ID_EDIT_A_1, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            hwndEditA2 = CreateWindow (TEXT ("edit"), NULL,
                         WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
                         335, 320, 70, 20,
                         hwnd, (HMENU) ID_EDIT_A_2, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;


            //Learn
            hWndButtonLearn = CreateWindowEx(
                0,
                "BUTTON",
                "Learn",
                WS_VISIBLE | WS_CHILD | WS_TABSTOP,
                20,
                360,
                110,
                40,
                hwnd,
                (HMENU) IDB_BUTTON_LEARN,
                hInstance,
                NULL);

            //Predict
            hWndButtonPredict = CreateWindowEx(
                0,
                "BUTTON",
                "Predict",
                WS_VISIBLE | WS_CHILD | WS_TABSTOP,
                155,
                360,
                110,
                40,
                hwnd,
                (HMENU) IDB_BUTTON_PREDICT,
                hInstance,
                NULL);

            //Optimize
            hWndButtonOptimize = CreateWindowEx(
                0,
                "BUTTON",
                "Optimize",
                WS_VISIBLE | WS_CHILD | WS_TABSTOP,
                290,
                360,
                110,
                40,
                hwnd,
                (HMENU) IDB_BUTTON_OPTIMIZE,
                hInstance,
                NULL);

            hdc = GetDC (hwnd) ;
            GetTextMetrics (hdc, &tm) ;
            
            hwndEditRuns = CreateWindow (TEXT ("edit"), NULL,
            WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
            340, 410, 60, 20,
            hwnd, (HMENU) ID_EDIT_RUNS, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            
            hwndEditCounter = CreateWindowEx (WS_EX_CLIENTEDGE, TEXT ("static"), (char *)perc,
            WS_CHILD | WS_VISIBLE | ES_READONLY,
            340, 435, 40, 20,
            hwnd, (HMENU) ID_EDIT_COUNTER, ((LPCREATESTRUCT) lParam) -> hInstance, NULL) ;
            
            return 0 ;




     case WM_COMMAND:

            switch (LOWORD(wParam))
            {

            case IDB_BUTTON_INPUT:
                GetOpenFilename("TXT files (*.txt)","*.txt");
                SendMessage(hwndEditInputFilename, WM_SETTEXT, 0, (LPARAM)InputFilename);
                return 0 ;

            case IDB_BUTTON_LEARN:
                 SetCursor(LoadCursor(NULL,IDC_WAIT));
                 ShowCursor(TRUE);
                 learn();
                 ShowCursor(FALSE);
                 SetCursor(LoadCursor(NULL,IDC_ARROW));
                 break;

            case IDB_BUTTON_PREDICT:
                 SetCursor(LoadCursor(NULL,IDC_WAIT));
                 ShowCursor(TRUE);
                 predict();
                 ShowCursor(FALSE);
                 SetCursor(LoadCursor(NULL,IDC_ARROW));
                 break;

            case IDB_BUTTON_OPTIMIZE:
                 SetCursor(LoadCursor(NULL,IDC_WAIT));
                 ShowCursor(TRUE);
                 optimize();
                 ShowCursor(FALSE);
                 SetCursor(LoadCursor(NULL,IDC_ARROW));
                 break;

            //Menu
            case IDM_FILE_OPEN:
                GetOpenFilename("TXT files (*.txt)","*.txt");
                SendMessage(hwndEditInputFilename, WM_SETTEXT, 0, (LPARAM)InputFilename);
                return 0 ;

            case IDM_APP_EXIT:
                SendMessage (hwnd, WM_CLOSE, 0, 0) ;
                return 0 ;
            case IDM_APP_HELP:
                MessageBox (hwnd, TEXT ("Help not yet implemented!"),
                                  szClassName, MB_ICONEXCLAMATION | MB_OK) ;
                return 0;
            case IDM_APP_ABOUT:
                MessageBox (hwnd, TEXT ("winSVM\n")
                                  TEXT ("(c) Martin Sewell, 2005"),
                                  szClassName, MB_ICONINFORMATION | MB_OK) ;
                return 0;
            }



        case WM_PAINT :


            hdc = BeginPaint (hwnd, &ps);
            SetBkColor (hdc, GetSysColor (COLOR_BTNFACE)) ;
            SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT)) ;

            TextOut (hdc, 20, 130, "C", 1);

            TextOut (hdc, 140, 260, "degree", 6);

            TextOut (hdc, 140, 280, "gamma", 5);

            TextOut (hdc, 140, 300, "a", 1);
            TextOut (hdc, 280, 300, "b", 1);

            TextOut (hdc, 140, 320, "gamma", 5);
            TextOut (hdc, 280, 320, "degree", 6);
            
            TextOut (hdc, 300, 410, "runs", 4);

            EndPaint (hwnd, &ps) ;
            return 0 ;

        case WM_SYSCOLORCHANGE:
            InvalidateRect (hwnd, NULL, TRUE) ;
            break ;


        case WM_DESTROY:
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;
        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
    }

}



